import { useEffect, useState } from "react"
import {
  ColumnDef,
  flexRender,
  getCoreRowModel,
  getPaginationRowModel,
  useReactTable,
} from "@tanstack/react-table"

import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from "@/components/ui/table"

import { DataTablePagination } from "@/Pages/Calls/components/data-table-pagination"
import TableSkeleton from "@/components/ui/table-skeleton"

interface DataTableProps<TData, TValue> {
  columns: ColumnDef<TData, TValue>[]
  data: TData[],
  isLoading: boolean,
  currentPage: number,
  itemsPerPage: number,
  setCurrentPage: React.Dispatch<React.SetStateAction<number>>,
  setItemsPerPage: React.Dispatch<React.SetStateAction<number>>,
  onPageChange: (direction: 'next' | 'prev', timestamp?: string) => void,
  calls: TData[]
}

export function DataTable<TData, TValue>({
  columns,
  data,
  isLoading,
  currentPage,
  itemsPerPage,
  setCurrentPage,
  setItemsPerPage,
  onPageChange,
  calls
}: DataTableProps<TData, TValue>) {
  const [rowSelection, setRowSelection] = useState({})
  const [pagination, setPagination] = useState({ pageIndex: 0, pageSize: 25 })

  const table = useReactTable({
    data,
    columns,
    state: {
      rowSelection,
      pagination,
    }, 
    columnResizeMode: "onChange",
    enableRowSelection: true,
    onRowSelectionChange: setRowSelection,
    manualPagination: true,
    onPaginationChange: setPagination,
    getPaginationRowModel: getPaginationRowModel(),
    getCoreRowModel: getCoreRowModel(),
  })


  const hideColumns = ['id']

  useEffect(() => {
    // Run this effect once to toggle column visibility on initial mount
    hideColumns.forEach((columnId) => {
      const column = table.getColumn(columnId)
      if (column) {
        column.toggleVisibility(false)
      }
    })
  }, [table])


  useEffect(() => {
    setCurrentPage(pagination.pageIndex + 1)
    setItemsPerPage(pagination.pageSize)
  }, [pagination])

  return (
    <div className="space-y-4">
      <div className="rounded-md border">
        <Table>
          <TableHeader>
            {table.getHeaderGroups().map((headerGroup) => (
              <TableRow key={headerGroup.id}>
                {headerGroup.headers.map((header) => {
                  return (
                    <TableHead key={header.id} colSpan={header.colSpan}>
                      {header.isPlaceholder
                        ? null
                        : flexRender(
                            header.column.columnDef.header,
                            header.getContext()
                          )}
                    </TableHead>
                  )
                })}
              </TableRow>
            ))}
          </TableHeader>
          <TableBody>
            {isLoading
              ? <TableSkeleton columnLength={columns.length} rowsLength={itemsPerPage} />
              : table.getRowModel().rows?.length ? (
                  table.getRowModel().rows.map((row) => (
                    <TableRow
                      key={row.id}
                      data-state={row.getIsSelected() && "selected"}
                    >
                      {row.getVisibleCells().map((cell) => (
                        <TableCell key={cell.id} style={{
                          width: `${cell.column.getSize()}px`,
                          maxWidth: `${cell.column.columnDef.maxSize}px`,
                          minWidth: `${cell.column.columnDef.minSize}px`,
                        }}>
                          {flexRender(
                            cell.column.columnDef.cell,
                            cell.getContext()
                          )}
                        </TableCell>
                      ))}
                    </TableRow>
                  ))
                ) : (
                  <TableRow>
                    <TableCell
                      colSpan={columns.length}
                      className="h-24 text-center"
                    >
                      No results.
                    </TableCell>
                  </TableRow>
                )
            }
          </TableBody>
        </Table>
      </div>
      <DataTablePagination 
        table={table} 
        itemsPerPage={itemsPerPage}
        currentPage={currentPage}
        setCurrentPage={setCurrentPage} 
        onPageChange={onPageChange}
        calls={calls}
      />
    </div>
  )
}
