import { getCallColumns } from "./components/columns.tsx"
import { DataTable } from "@/Pages/Calls/components/data-table.tsx"
import React, { useCallback, useEffect, useMemo, useRef, useState } from "react"
import { Call } from "./components/data.ts"
import { Tabs, TabsList, TabsTrigger } from "@/components/ui/tabs"
import { toast } from "sonner"
import { Button } from "@/components/ui/button.tsx"
import { cn } from "@/lib/utils.ts"
import { RefreshCcw } from "lucide-react"
import axios from 'axios'


export default function CallsPage() {
  const [calls, setCalls] = useState<Call[]>([])
  const [currentPage, setCurrentPage] = useState(1)
  const [itemsPerPage, setItemsPerPage] = useState(10)
  const [showLoadingAnimation, setShowLoadingAnimation] = useState(false)
  const [fetchingInBackground, setFetchingInBackground] = useState(false)

  const fetchCalls = useCallback(async (showAnimation: boolean = false, direction?: 'next' | 'prev',
    timestamp?: string) => {
    try {
      console.log("Called with ", showAnimation, direction, timestamp)
      if (showAnimation) setShowLoadingAnimation(true);
      if (!showAnimation) setShowLoadingAnimation(false);

      const queryParams = new URLSearchParams()

      queryParams.append("limit", itemsPerPage.toString())
      // queryParams.append("offset", (itemsPerPage*(currentPage-1)).toString())

      if (timestamp && direction) {
        if (direction === 'next') {
          queryParams.append("direction", 'next')
          queryParams.append("timestamp", timestamp)
        } else {
          queryParams.append("direction", 'prev')
          queryParams.append("timestamp", timestamp)
        }
      }

      const query = axios.get(`${import.meta.env.VITE_BASE_API_URL}/user/call?${queryParams.toString()}`, {
        headers: {
          Authorization: localStorage.getItem('token') as string
        }
      })

      const [response] = await Promise.all([query, new Promise(res => setTimeout(res, showAnimation ? 1000 : 0))])
      
      console.log("Response", response)
      const data = await response.data
      setCalls(data)


      setFetchingInBackground(true)
    } catch (error) {
      toast.error("Error", {
        description: "Failed to fetch call data, please try again!",
        action: {
          label: "Close",
          onClick: () => console.log("Close"),
        },
      })
      console.log(error)
      setFetchingInBackground(false)
    } finally {
      setShowLoadingAnimation(false);
    }
  }, [itemsPerPage]);

  // Add this handler function
const handlePageChange = (direction: 'next' | 'prev', timestamp?: string) => {
  fetchCalls(true, direction, timestamp);
}


  useEffect(() => {
    fetchCalls(true);
    
    // const interval = setInterval(() => {
    //     fetchCalls(false)
    // }, 15000)


    // return () => {
    //   clearInterval(interval)
    // }
  }, [fetchCalls])


  const columns = useMemo(() => getCallColumns(), [])

 
  return (
    <>
        <div className="flex items-start justify-between">
          <h1 className="text-2xl font-bold">Calls</h1>
          <div className="flex items-center justify-center space-x-3">
              <Button size="sm" variant="outline" className="mt-1" onClick={() => fetchCalls(true)}>
                <RefreshCcw className="scale-90" />
              </Button>
            <span className="relative inline-flex mt-1">
              {
                fetchingInBackground
                  ? <Button size="sm" className="px-4 text-green-700" variant="secondary">Live</Button>
                  : <Button size="sm" className="px-4 text-red-500" variant="secondary">Stale Data</Button>
              }
              <span className="flex absolute h-3 w-3 top-0 right-0 -mt-1 -mr-1">                
                <span
                  className={cn(
                    "animate-ping absolute inline-flex h-full w-full rounded-full opacity-75",
                    fetchingInBackground ? "bg-green-400" : "bg-red-400",  // Conditionally change the background color
                  )}
                ></span>
                <span
                  className={cn(
                    "relative inline-flex rounded-full h-3 w-3",
                    fetchingInBackground ? "bg-green-500" : "bg-red-500"  // Conditionally change the background color
                  )}
                ></span>
              </span>
            </span>
          </div>
        </div>
        <DataTable data={calls} columns={columns} isLoading={showLoadingAnimation} currentPage={currentPage} itemsPerPage={itemsPerPage} setCurrentPage={setCurrentPage} setItemsPerPage={setItemsPerPage} onPageChange={handlePageChange}
  calls={calls} />
    </>
  )
}