import { useState } from "react";
import { toast } from "sonner";
import {
  Card,
  CardHeader,
  CardTitle,
  CardDescription,
  CardContent,
} from "@/components/ui/card";
import { Label } from "@/components/ui/label";
import { Input } from "@/components/ui/input";
import { Button } from "@/components/ui/button";
import { useNavigate } from "react-router-dom";
import { User } from "@/types";
import axios from "axios";

type LoginPageProps = {
  setShowLoginPage: React.Dispatch<React.SetStateAction<boolean>>;
  setUser: React.Dispatch<React.SetStateAction<User>>;
};

function LoginPage({ setShowLoginPage, setUser }: LoginPageProps) {
  const [email, setEmail] = useState("");
  const [password, setPassword] = useState("");
  const [loading, setShowLoading] = useState(false);
  const navigate = useNavigate();

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();

    if (!email || !password) {
      toast.error("Error", {
        description: "Email and Password are required!",
        action: {
          label: "Close",
          onClick: () => console.log("Close"),
        },
      });
      return;
    }

    try {
      setShowLoading(true);
      const body = JSON.stringify({email, password})
      const response = await axios.post(`${import.meta.env.VITE_BASE_API_URL}/user/login`, body, {
        headers: {'Content-Type': 'application/json'}
      })
      
      const data =  response.data;
      console.log(data)
      localStorage.setItem("token", data.token);
      localStorage.setItem("email", data.email);
      localStorage.setItem("role", data.role);
      setUser({
        token: data.token,
        email: data.email,
        role: data.role
      })
      setShowLoginPage(false)
      navigate('/calls')

    } catch (err) {
      if (axios.isAxiosError(err) && err.response?.status === 401) {
        toast.error("Invalid Credentials", {
          description: "The email or password you entered is incorrect.",
          action: {
            label: "Close",
            onClick: () => console.log("Close"),
          },
        });
      } else {
        toast.error("Error", {
          description: "Login Failed, please try again!",
          action: {
            label: "Close",
            onClick: () => console.log("Close"),
          },
        });
      }
    } finally {
      setShowLoading(false);
    }
  };

  return (
    <form
      onSubmit={handleSubmit}
      className="w-screen h-screen flex justify-center items-center"
    >
      <Card className="mx-auto max-w-sm">
        <CardHeader className="space-y-1">
          <CardTitle className="text-2xl font-bold">Login</CardTitle>
          <CardDescription>
            Enter your email and password to login to your account
          </CardDescription>
        </CardHeader>
        <CardContent>
          <div className="space-y-4">
            <div className="space-y-2">
              <Label htmlFor="email">Email</Label>
              <Input
                id="email"
                type="email"
                onChange={(e) => setEmail(e.target.value)}
                placeholder="m@example.com"
                required
              />
            </div>
            <div className="space-y-2">
              <Label htmlFor="password">Password</Label>
              <Input
                id="password"
                onChange={(e) => setPassword(e.target.value)}
                type="password"
                required
              />
            </div>
            <Button type="submit" className="w-full">
              {!loading ? "Login" : "Logging in"}
              {loading && (
                <svg
                  className="animate-spin ml-1 mr-3 h-5 w-5 text-white"
                  xmlns="http://www.w3.org/2000/svg"
                  fill="none"
                  viewBox="0 0 24 24"
                >
                  <circle
                    className="opacity-25"
                    cx="12"
                    cy="12"
                    r="10"
                    stroke="currentColor"
                    strokeWidth="4"
                  ></circle>
                  <path
                    className="opacity-75"
                    fill="currentColor"
                    d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"
                  ></path>
                </svg>
              )}
            </Button>
          </div>
        </CardContent>
      </Card>
    </form>
  );
}

export default LoginPage;
